/*
 * #%L
 * EUGene :: EUGene
 * 
 * $Id: ObjectModelTransformerToJava.java 863 2010-04-15 14:22:49Z tchemit $
 * $HeadURL: http://svn.nuiton.org/svn/eugene/tags/eugene-2.0.1/eugene/src/main/java/org/nuiton/eugene/java/ObjectModelTransformerToJava.java $
 * %%
 * Copyright (C) 2004 - 2010 CodeLutin
 * %%
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as 
 * published by the Free Software Foundation, either version 3 of the 
 * License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Lesser Public License for more details.
 * 
 * You should have received a copy of the GNU General Lesser Public 
 * License along with this program.  If not, see
 * <http://www.gnu.org/licenses/lgpl-3.0.html>.
 * #L%
 */

package org.nuiton.eugene.java;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.nuiton.eugene.Template;
import org.nuiton.eugene.models.object.*;

/**
 * OM2JavaTransformer
 * <p/>
 * Created: 28 oct. 2009
 *
 * @author fdesbois
 * @version $Revision: 863 $
 *          <p/>
 *          Mise a jour: $Date: 2010-04-15 16:22:49 +0200 (jeu., 15 avril 2010) $
 *          par : */
public abstract class ObjectModelTransformerToJava extends ObjectModelTransformer<ObjectModel> {

    private static final Log log =
            LogFactory.getLog(ObjectModelTransformerToJava.class);
    /**
     * internal builder
     */
    protected JavaBuilder builder;

    @Override
    protected Template<ObjectModel> initOutputTemplate() {
        // by default, use a stupid java generator
        return new JavaGenerator();
    }

    @Override
    protected ObjectModel initOutputModel() {
        if (log.isDebugEnabled()) {
            log.debug("inputModelName = " + getModel().getName());
        }
        builder = new JavaBuilder(getModel().getName());
        return builder.getModel();
    }

    @Override
    protected void debugOutputModel() {
        if (log.isDebugEnabled()) {
            log.debug("classes : " + getOutputModel().getClasses().size());
            for (ObjectModelClass clazz : getOutputModel().getClasses()) {
                log.debug("  class : " + clazz.getQualifiedName());
                ImportsManager manager = getOutputModel().getExtension(
                        clazz.getQualifiedName(), ImportsManager.class
                );
                for (String imports :
                        manager.getImports(clazz.getPackageName())) {
                    log.debug("  import : " + imports);
                }
                for (ObjectModelAttribute attribute : clazz.getAttributes()) {
                    log.debug("  attribute : " + attribute.getType() +
                              " " + attribute.getName());
                }
                for (ObjectModelOperation operation : clazz.getOperations()) {
                    log.debug("  operation : " + operation.getReturnType() +
                              " " + operation.getName());
                }
            }
        }
    }

    protected ObjectModelClass createClass(String name, String packageName) {
        return builder.createClass(name, packageName);
    }

    public ObjectModelEnumeration createEnumeration(String name,
                                                    String packageName) {
        return builder.createEnumeration(name, packageName);
    }

    protected ObjectModelClass createAbstractClass(String name,
                                                   String packageName) {
        return builder.createAbstractClass(name, packageName);
    }

    protected ObjectModelInterface createInterface(String name,
                                                   String packageName) {
        return builder.createInterface(name, packageName);
    }

    protected void setSuperClass(ObjectModelClass classifier,
                                 String superclassQualifiedName) {
        builder.setSuperClass(classifier, superclassQualifiedName);
    }

    protected void setSuperClass(ObjectModelClass classifier,
                                 Class<?> superclassQualifiedName) {
        builder.setSuperClass(classifier, superclassQualifiedName.getName());
    }

    protected void addInterface(ObjectModelClassifier classifier,
                                String interfaceQualifiedName) {
        builder.addInterface(classifier, interfaceQualifiedName);
    }

    protected void addInterface(ObjectModelClassifier classifier,
                                Class<?> interfaceQualifiedName) {
        builder.addInterface(classifier, interfaceQualifiedName.getName());
    }

    protected void addImport(ObjectModelClassifier classifier,
                             String imports) {
        builder.addImport(classifier, imports);
    }

    protected void addImport(ObjectModelClassifier classifier,
                             ObjectModelClass imports) {
        builder.addImport(classifier, imports.getPackageName() + "." + imports.getName());
    }

    protected void addImport(ObjectModelClassifier classifier,
                             Class<?> imports) {
        builder.addImport(classifier, imports.getName());
    }

    public String getConstantName(String propertyName) {
        return builder.getConstantName(propertyName);
    }

    protected ObjectModelAttribute addConstant(
            ObjectModelClassifier classifier,
            String name,
            String type,
            String value,
            ObjectModelModifier visibility) throws IllegalArgumentException {
        return builder.addConstant(classifier, name, type, value, visibility);
    }

    protected ObjectModelAttribute addConstant(
            ObjectModelClassifier classifier,
            String name,
            Class<?> type,
            String value,
            ObjectModelModifier visibility) throws IllegalArgumentException {
        return builder.addConstant(classifier, name, type.getName(), value, visibility);
    }

    protected ObjectModelAttribute addAttribute(
            ObjectModelClassifier classifier,
            String name,
            String type,
            String value,
            ObjectModelModifier... modifiers) {
        return builder.addAttribute(classifier, name, type, value, modifiers);
    }

    protected ObjectModelAttribute addAttribute(
            ObjectModelClassifier classifier,
            String name,
            Class<?> type,
            String value,
            ObjectModelModifier... modifiers) {
        return builder.addAttribute(classifier, name, type.getName(), value, modifiers);
    }

    protected ObjectModelAttribute addAttribute(
            ObjectModelClassifier classifier,
            ObjectModelAttribute attribute) {
        return builder.addAttribute(classifier, attribute);
    }

    public ObjectModelAttribute addAttribute(
            ObjectModelClassifier classifier,
            String name,
            String type) {
        return builder.addAttribute(classifier, name, type);
    }

    protected ObjectModelOperation addConstructor(
            ObjectModelClass clazz, ObjectModelModifier visibility)
            throws IllegalArgumentException {
        return builder.addConstructor(clazz, visibility);
    }

    protected ObjectModelOperation addConstructor(
            ObjectModelEnumeration clazz, ObjectModelModifier visibility)
            throws IllegalArgumentException {
        return builder.addConstructor(clazz, visibility);
    }

    protected ObjectModelOperation addOperation(
            ObjectModelClassifier classifier,
            String name,
            String type,
            ObjectModelModifier... modifiers) {
        return builder.addOperation(classifier, name, type, modifiers);
    }

    protected ObjectModelOperation addOperation(
            ObjectModelClassifier classifier,
            String name,
            Class<?> type,
            ObjectModelModifier... modifiers) {
        return builder.addOperation(classifier, name, type == null ? null : type.getName(), modifiers);
    }

    protected void addLiteral(ObjectModelEnumeration classifier, String name) {
        builder.addLiteral(classifier, name);
    }

    protected ObjectModelParameter addParameter(
            ObjectModelOperation operation,
            String type,
            String name) {
        return builder.addParameter(operation, type, name);
    }

    protected ObjectModelParameter addParameter(
            ObjectModelOperation operation,
            Class<?> type,
            String name) {
        return builder.addParameter(operation, type.getName(), name);
    }

    protected void addException(
            ObjectModelOperation operation,
            String exception) {
        builder.addException(operation, exception);
    }

    protected void addException(
            ObjectModelOperation operation,
            Class<?> exception) {
        builder.addException(operation, exception.getName());
    }

    protected void setOperationBody(ObjectModelOperation operation,
                                    String body)
            throws IllegalArgumentException {
        builder.setOperationBody(operation, body);
    }

    public ObjectModelClassifier addInnerClassifier(
            ObjectModelClass clazz,
            ObjectModelType type,
            String name) throws IllegalArgumentException {
        return builder.addInnerClassifier(clazz, type, name);
    }

    public void setDocumentation(ObjectModelElement element,
                                 String documentation) {
        builder.setDocumentation(element, documentation);
    }

    public void addAnnotation(ObjectModelClassifier classifier,
                              ObjectModelElement element,
                              String annotation) {
        builder.addAnnotation(classifier, element, annotation);
    }

    public ObjectModelOperation addBlock(ObjectModelClassifier classifier,
                                         ObjectModelModifier... modifiers) {
        return builder.addBlock(classifier, modifiers);
    }
}
